// Copyright (C) 2006-2007 Johan Jansson and Anders Logg.
// Licensed under the GNU LGPL Version 2.1.
//
// First added:  2006-02-07
// Last changed: 2007-08-20
//
// This demo program solves the equations of static
// linear elasticity for a gear clamped at two of its
// ends and twisted 30 degrees.

#include <dolfin.h>
#include "NeoHookean.h"

using namespace dolfin;

class Clamp : public Function, public TimeDependent
{
public:
  Clamp(Mesh& mesh, real& t) : Function(mesh),TimeDependent(t) {}

  void eval(real* values, const real* x) const
  {
    values[0] = 0.0;
    values[1] = 0.0;
  }

};

// Sub domain for clamp at left end
class Left : public SubDomain
{
  bool inside(const real* x, bool on_boundary) const
  {
    return (std::abs(x[0]) < DOLFIN_EPS) && on_boundary;
  }
};


// Specified pull force at right end
class PullForce : public Function, public TimeDependent
{
public:

    PullForce(Mesh& mesh, const real& t) : Function(mesh),TimeDependent(t) {}

    void eval(real* values, const real* x) const
    {
      if (std::abs(x[0] - 1.0) < DOLFIN_EPS)
          values[0] = 0.1*time();
      else
          values[0] = 0.0;
      values[1] = 0.0;
    }
};


int main(int argc, char* argv[])
{
  dolfin_init(argc, argv);
  // Pseudo time
  real t = 0.0;

  // Create mesh
  UnitSquare mesh(16,16); 

  // Create right-hand side
  Function f(mesh, 0.0);
  PullForce g(mesh, t);

  // Set up boundary condition at left end
  Clamp c(mesh, t);
  Left left;
  DirichletBC bc(c, mesh, left);

  // Solution function
  Function u;

  // Create forms and nonlinear PDE
  NeoHookeanBilinearForm a(u);
  NeoHookeanLinearForm L(u, f, g);
  NonlinearPDE pde(a, L, mesh, bc);

  // Solve nonlinear problem in a series of steps
  real dt = 0.1; real T  = 1.0;

  //pde.dolfin_set("Newton relative tolerance", 1e-2); 
  //pde.dolfin_set("Newton convergence criterion", "incremental"); 

  // Solve
  pde.solve(u, t, T, dt);

  // Extract solution
  Function disp = u[0];
  Function pr = u[1];

  // Plot solution
  plot(disp, "displacement");

  // Save solution to VTK format
  File vtk_file("neoHookean.pvd");
  vtk_file << disp;

  // Save solution to XML format
  File xml_file("neoHookean.xml");
  xml_file << disp;

  return 0;
}
